package Dummy3;
use strict;
use warnings FATAL => 'all';


sub test {
    my $val = "hello";
    my $val2 = "__END__";
}

my $t = sub {
  my $hello = 1;
};

=head3 TEstero

Bla Bla Bla

=cut

1;

=head3 Testero2

Bla Bla no end `=cut`